function [cal] = ISR_Calibration_subphase (cal)

% This function performs the calibration of the ISR and takes the subphase
% into account (procedure proposed by Tajuelo et al. 2015).
%
%                                                                   2018 DR



%% Measured data during calibration

omegaV_Hz_all = cal.data.CurrentFrequency_Hz_; % frequency of voltage in Hz
delta_all = cal.data.MeasuredDelta_deg_;       % store the phase angle in deg
AR_all = cal.data.AmplitudeRatio_pixel_I_;     % amplitude Ratio in pixel/I


% Look for repeated measurements and calculate mean and std if done so
[omegaHz,IA] = unique(omegaV_Hz_all,'stable');
IA=[IA;length(omegaV_Hz_all)+1];

AR_experiment = NaN(length(omegaHz),1);
AR_experiment_std = AR_experiment;
delta_experiment = AR_experiment_std;
delta_experiment_std = delta_experiment;

for i=1:length(omegaHz)
    AR_experiment(i) = mean(AR_all(IA(i):IA(i+1)-1));
    AR_experiment_std(i) = std(AR_all(IA(i):IA(i+1)-1));
    
    delta_experiment(i) = mean(delta_all(IA(i):IA(i+1)-1));
    delta_experiment_std(i) = std(delta_all(IA(i):IA(i+1)-1));
end


omegarads = omegaHz*2*pi; % frequency of voltage in rad/s




%% Experimental parametersParameters 

Channelradius = (cal.parameters.ChannelWidth_mm_(1)/2 ) *10^(-3); %m
Needleradius = (cal.parameters.NeedleDiameter_mm_(1)/2) *10^(-3); %m
NeedleLength = (cal.parameters.NeedleLength_mm_(1)) *10^(-3); %m
NeedleMass = (cal.parameters.NeedleMass_g_(1)) *10^(-3); % kg


rho_b = (cal.parameters.SubphaseDensity_g_ml_(1)) * 10^(3); % kg/m^3
eta_b = (cal.parameters.SubphaseBulkViscosity_mPas_(1)) *10^(-3); % Pa.s

% calculate the Reynolds number
Re = rho_b * omegarads * Needleradius^2/eta_b;

% calculate lambda and the channel parameter
lambda = Channelradius/Needleradius; % R_channel / r_needle
channelparameter = log(lambda); % = max p = log (R/a)

%% create folder and path to store figures & data

if exist(fullfile(cal.path,'Analysis'),'dir')
    if exist(fullfile(cal.path,'Analysis','Calibration'),'dir')
    else
        mkdir(fullfile(cal.path,'Analysis','Calibration'))
    end
else
    mkdir(fullfile(cal.path,'Analysis'))
    mkdir(fullfile(cal.path,'Analysis','Calibration'))
end

SavePath = fullfile(fullfile(cal.path,'Analysis','Calibration'));

%% Define technical variables

% for large mesh size (> 64) the use of sparse matrices saves a lot of time
ThM = 128;   % mesh size of theta      (ThM+1 points)
PM = ThM; % mesh size of p = ln(r/a) (PM+1 points)

dTh = (pi/2) / ThM; % difference in theta which goes from 0 to pi/2: delta Theta
dP = channelparameter / PM; % difference in p which goes from 0 to log(R/r): delta p



velocityfield=struct('g',[],'analytical',[]);
h = waitbar(0,'Calculating numeric results');

K=struct('y_analytical',[],'y_numerical',[],'z_numerical',[]);



for jj=1:length(Re)
    %% Begin with the formalism:
    
    
    % initialise the mesh matrix and the boundary conditions for Bo = 0
    
    % A contains all g(theta, p) with
    %       A(1, 1         : ThM+1)   = g(0:pi/2 , 0)
    %       A(1, ThM+2     : 2*(ThM+1)) = g(0:pi/2 , 1)
    %       A(1, 2*(ThM+2) : 3*(ThM+1)) = g(0:pi/2 , 2)
    %       etc.
    
    % Define A0 and b
    %A0 = ((eye ((ThM+1)*(PM+1))));
    b=(zeros((ThM+1)*(PM+1),1));
    b(1:ThM+1)=1; % BC at p=0 (eq. 23a)
    % create a placeholder for getting the right components to create the
    % sparse matrix A
    IDX = struct('theta_interface',[],'theta_z',[],'bulk',[]);
    
    for k=((ThM+1)+1):((PM*(ThM+1)))
        
        % This is the number by which dP has to be multiplied in order to equal the p in the paper
        %p=ceil(k/(ThM+1))-1;
        
        % If loop according to the boundary conditions
        if ceil(k/(ThM+1))==floor(k/(ThM+1)) %
            % this case coresponds to the index of A for theta = pi/2 which is
            % the interface plane and corresponds to eq. 24
            % --> g(pi/2,p) = g(pi/2-dTh,p)
            
            % "normal" code
%             A0(k,k)          = (1i*Re(jj)*exp(2*p*dP)  +  2/dP^2  +  2/dTh^2);
%             A0(k,k+(ThM+1))  = -1/dP^2;
%             A0(k,k-(ThM+1))  = -1/dP^2;
%             A0(k,k-1)        = -2/dTh^2;

            IDX.theta_interface = [IDX.theta_interface ;k];
            
            
        elseif ceil((k-1)/(ThM+1))==floor((k-1)/(ThM+1))
            % this case corresponds to the index of A for theta = 0 which is
            % the vertical (z axis) and corresponds to eq. 23c
            % --> dg/dtheta |0 = 0
            
%             A0(k,k)         = (1i*Re(jj)*exp(2*p*dP)  +  2/dP^2  +  2/dTh^2);
%             A0(k,k+1)       = -2/dTh^2;
%             A0(k,k+(ThM+1)) = -1/dP^2;
%             A0(k,k-(ThM+1)) = -1/dP^2;
            
            IDX.theta_z = [IDX.theta_z ;k];
            
        else
            % this case corresponds to the bulk equation (eq. 22)
            
%             A0(k,k)          = (1i*Re(jj)*exp(2*p*dP)   +  2/dP^2   +  2/dTh^2);
%             A0(k,k+1)        = -1/dTh^2;
%             A0(k,k-1)        = -1/dTh^2;
%             A0(k,k+(ThM+1))  = -1/dP^2;
%             A0(k,k-(ThM+1))  = -1/dP^2;
            
            IDX.bulk = [IDX.bulk ;k];
            
        end
    end
    
    % use sparse matrices in order to allow for large mesh sizes.
    A_s_0 = sparse(1:(ThM+1)*(PM+1),1:(ThM+1)*(PM+1),1,(ThM+1)*(PM+1),(ThM+1)*(PM+1));
    
    A_s_I1 = sparse(IDX.theta_interface,IDX.theta_interface,(1i*Re(jj)*exp(2*(ceil(IDX.theta_interface/(ThM+1))-1)*dP)  +  2/dP^2  +  2/dTh^2-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_I2 = sparse(IDX.theta_interface,IDX.theta_interface+(ThM+1),(-1/dP^2),                              ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_I3 = sparse(IDX.theta_interface,IDX.theta_interface-(ThM+1),(-1/dP^2),                              ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_I4 = sparse(IDX.theta_interface,IDX.theta_interface-1      ,(-2/dTh^2),                             ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    
    
    
    A_s_Z1 = sparse(IDX.theta_z,IDX.theta_z,(1i*Re(jj)*exp(2*(ceil(IDX.theta_z/(ThM+1))-1)*dP)  +  2/dP^2  +  2/dTh^2-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_Z2 = sparse(IDX.theta_z,IDX.theta_z+1,      (-2/dTh^2),                             ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_Z3 = sparse(IDX.theta_z,IDX.theta_z+(ThM+1),(-1/dP^2),                              ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_Z4 = sparse(IDX.theta_z,IDX.theta_z-(ThM+1),(-1/dP^2),                              ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    
    
    
    A_s_B1 = sparse(IDX.bulk,IDX.bulk,(1i*Re(jj)*exp(2*(ceil(IDX.bulk/(ThM+1))-1)*dP)   +  2/dP^2   +  2/dTh^2-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_B2 = sparse(IDX.bulk,IDX.bulk+1,(-1/dTh^2),                                     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_B3 = sparse(IDX.bulk,IDX.bulk-1,(-1/dTh^2),                                     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_B4 = sparse(IDX.bulk,IDX.bulk+(ThM+1),(-1/dP^2),                                ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_B5 = sparse(IDX.bulk,IDX.bulk-(ThM+1),(-1/dP^2),                                ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    
    
    
    A_sparse = A_s_0+...
        A_s_I1+A_s_I2+A_s_I3+A_s_I4+...
        A_s_Z1+A_s_Z2+A_s_Z3+A_s_Z4+...
        A_s_B1+A_s_B2+A_s_B3+A_s_B4+A_s_B5;
    
    
    
    % Step 1: Calculation of the velocity field g with Bo* = 0 !
    
    % Inverse A to get the velocity field g
    g = (mldivide(A_sparse,sparse(b)));
    velocityfield(jj).g=full(g);
    
    % get the relevant components for plotting
    K(jj).y_numerical = velocityfield(jj).g((1:PM+1)*(ThM+1))./velocityfield(jj).g(1);
    K(jj).z_numerical = velocityfield(jj).g(1:(ThM+1):end)   ./velocityfield(jj).g(1);
    
    % update the waitbar
    waitbar(jj/length(Re),h,...
        ['calculating numerical result: ' num2str(100* (jj/length(Re)),'%.0f') '%']);
    
end
close(h);

% phi contains the values of the x-axis p-r/R-r where p = [a,R]
phi = (exp(linspace(0,channelparameter,PM+1))* Needleradius - Needleradius) ./ (Channelradius -Needleradius);








%% Analytical solution for Bo = 0; (eq. 17)
% define coefficients
Channelheight = Channelradius; % h/a
summing = 10000; % summing over 10000 elements
Z=0;

% this is the parambeter lambda (eq. 8)
lambda_n=@(n) ( (2*n-1)*pi*Needleradius)/(2*Channelradius);

h = waitbar(0,'Calculating analytical result');
% take the same y-values for the analytical calculation
Y = (phi.*(Channelradius -Needleradius)+Needleradius)/Needleradius  ;% = y/a

% loop over all frequencies
for jj = 1:length(Re)
    % define eq. 17 with Bo = 0 and Z = 0
    V_n = @(Y,Z,n) ((1/lambda)* cos(lambda_n(n)*Y)./sqrt(lambda_n(n)^2+1i*Re(jj))        .* tanh(Channelheight/Needleradius*sqrt(lambda_n(n)^2+1i*Re(jj))));
    
    % loop over the mesh
    for j=1:length(Y)
        
        V_n_it = NaN(summing,1);
        % create the sum over 10'000 terms
        for i = 1:summing
            V_n_it(i)=V_n(Y(j),Z,i);
        end
        waitbar( jj/length(Re),h,...
            ['calculating analytical result: ' num2str(100*(jj/length(Re)),'%.0f') '%']);
        velocityfield(jj).analytical(j) = sum(V_n_it);
    end
    
    theta_analytical= angle(velocityfield(jj).analytical(1)); % angle(V(0,0))
    % calculate k (eq. 18)
    K(jj).y_analytical = velocityfield(jj).analytical*exp(-1i*theta_analytical)./abs(velocityfield(jj).analytical(1));
    
end
close(h);



% Calculate the maximal error between analytical and numerical solution

er=NaN(length(Re),1);
for jj = 1:length(Re)
    
    % error = mean ( |K_analytical- K_numerical| ./sqrt (1+ Imag(K_analytical) ) 
    er(jj) =  mean(abs((K(jj).y_analytical(2:end-1)')-K(jj).y_numerical(2:end-1)))/sqrt(1+(max(imag(K(jj).y_analytical(2:end-1)).^2)));
end


% plot error
figure(1)
loglog(omegaHz,er,'--o','Linewidth',2,'MarkerSize',8)
set(gca,'FontSize',16)
title('Error between analytical and numerical solution')
xlabel('frequency [Hz]')
ylabel('1/N \Sigma ( |k_{analytic}-k_{FD }|) / max k_{analytic} ')



savefig(1,      fullfile(SavePath,'Subphas_error_analytical_numerical.fig'))
print(1,'-dpng',fullfile(SavePath,'Subphas_error_analytical_numerical.png'))






%% plot the calculated velocity fields along the interface (y,z=0) and bulk (y=0,z)



for jj=1:length(Re)
    
    % create the figure which is change lateroin
    figure(2)
    %velocity profile
    plot(phi,real(K(jj).z_numerical),'-vc','MarkerFaceColor','c');hold on
    plot(phi,imag(K(jj).z_numerical),'-^m','MarkerFaceColor','m')
    
    plot(phi,real(K(jj).y_numerical),'-ob');
    plot(phi,imag(K(jj).y_numerical),'-sr');
    plot(phi,real(K(jj).y_analytical),'--k','Linewidth',2)
    plot(phi,imag(K(jj).y_analytical),'--g','Linewidth',2)%;hold off
    xlim([0 1]);
    ylim([-inf,1]);
    xlabel('\phi = (r-a) / (R-a)')
    ylabel('k(\phi) = g(p,\theta) / g(r=0,\theta=0)')
    legend('Real(k(\phi, \theta = 0))','Imag(k(\phi, \theta = 0))',...
        'Real(k(\phi, \theta = \pi/2))','Imag(k(\phi, \theta = \pi/2))',...
        'Real(analytical(\theta = \pi/2))','Imag(analytical(\theta = \pi/2))')
    title(['\omega = ' num2str(omegaHz(jj)) ' Hz \rightarrow Re = ' num2str(Re(jj),'%.2e')]); hold off
    set(gca,'fontsize',16)
    drawnow
    
    savefig(2,      fullfile(SavePath,['Subphase_K_Re_' num2str(jj) '.fig']))
    print(2,'-dpng',fullfile(SavePath,['Subphase_K_Re_' num2str(jj) '.png']))
    
    
end

%% Step 2: Calculation of subphase drag on the needle from the calculated velocity field g

% PARAM is used for calculating the new amplitude ratio
PARAM  = 1i*2*NeedleLength*omegarads * eta_b;

% INTEGRAL = int ( -dg/dp at p=0 over dTheta)
INTEGRAL =NaN(length(omegaHz),1);
for jj = 1:length(omegaHz)
    % store for convenience
    g=velocityfield(jj).g;
    
    % Drag force
    
    % DERIVATIVE =  -dg/dp at p=0 and theta = pi/2 --> not needed if Bo = 0
    %DERIVATIVE =-( -3+4*g(2*(ThM+1))-g(3*(ThM+1)) )/(2*dP); % g(ThM+1)=1 because of BC
    
    % INTEGRAL = int ( -dg/dp at p=0 over dTheta)
    INTEGRAL(jj) = -sum( -3 + 4*g((ThM+1)+1:2*(ThM+1)) - g(2*(ThM+1)+1:3*(ThM+1)) ) *dTh/(2*dP); %g(1:ThM+1)=1 because of BC
end

% define the function F/z_calculated dependent on E and K
Fz_calculated =@(E,K)   E *(PARAM .* INTEGRAL  +  (  K  -  NeedleMass*(omegarads.^2)) );



%% Step 3: Comparison of the amplitude ratios --> find E and K

% I don't know if these parameters are read correctly
options = optimset('TolFun',1e-6,'MaxFunEvals',1000,'MaxIter',1000,'PlotFcns',@optimplotfval);

% perform fminsearch for finding E and K such that 
[a, val, exitflag]=fminsearch(@Calibration_Minimizing,[10,0],options,PARAM,INTEGRAL,NeedleMass,(omegarads),(AR_experiment));

% incase K is negative, start again with new initial conditions according
% to the last result for E (a(1)).
while a(2)<0 || a(1)<0
    if a(2)<0
    [a, val, exitflag]=fminsearch(@Calibration_Minimizing,[a(1),0],options,PARAM,INTEGRAL,NeedleMass,(omegarads),(AR_experiment));
    else
    [a, val, exitflag]=fminsearch(@Calibration_Minimizing,[0,a(2)],options,PARAM,INTEGRAL,NeedleMass,(omegarads),(AR_experiment));   
    end
end

% S_ofEK = @(a) max((abs(1./abs(a(1) *(PARAM.*INTEGRAL + (a(2)-NeedleMass*(omegarads.^2)) )) - AR_experiment)));
% A = [];
% b = [];
% Aeq = [];
% beq = [];
% lb = zeros(1,2);
% ub = [];
%[a, val, exitflag]=fmincon(S_ofEK,[10,0],A,b,Aeq,beq,lb,ub);

E = a(1);
K_compliance = a(2);


% plot S(E,K) = F/z_calculated(E,K) - F/z_measured
%%
Emesh= linspace((a(1))-10,(a(1))+10,201);
Kmesh= logspace(log10(a(2))-1,log10(a(2))+1,201);

S=NaN(length(Emesh),length(Kmesh));

for i=1:length(Emesh)
    for j=1:length(Kmesh)
       %S(i,j)= S_ofEK([Emesh(i),Kmesh(j)]);
       S(i,j)= Calibration_Minimizing([Emesh(i),Kmesh(j)],PARAM,INTEGRAL,NeedleMass,omegarads,AR_experiment);
    end
end
figure(4)
subplot(1,3,[1 2])
mesh(Emesh,Kmesh,log10(S)');hold on
plot3(a(1),a(2),log10(Calibration_Minimizing(a,PARAM,INTEGRAL,NeedleMass,omegarads,AR_experiment)),'or','MarkerFaceColor','r','MarkerSize',10); hold off
%plot3(a(1),a(2),log10(S_ofEK(a)),'or','MarkerFaceColor','r','MarkerSize',10); hold off
%set(gca,'XScale','log')
set(gca,'YScale','log')

xlabel('E')
ylabel('K')
zlabel('log (S(E,K))')
title(['E = ' num2str(E,'%.2f') ' (A/N)(m/Pixel), K = ' num2str(K_compliance,'%.2e') ' N/m'])

set(gca,'FontSize',18)

subplot(1,3,3)
loglog(omegaHz,  (K_compliance-NeedleMass.*omegarads.^2),'-ko',...
       omegaHz,  (imag(PARAM.*INTEGRAL))     ,'--rv'); hold on

plot(omegaHz,  -(real(PARAM.*INTEGRAL))     ,'--bs','MarkerFaceColor','b');
plot(omegaHz,  -(K_compliance-NeedleMass.*omegarads.^2),'-ko','MarkerFaceColor','k');hold off
   
ylabel('F/z [N/m]')
xlabel('\omega [Hz]')
legend('K-m\omega^2','Re D_{sub}','Im D_{sub}')
title('Components of F/z. empty = positive, filled = negative')
set(gca,'FontSize',18)

savefig(4,      fullfile(SavePath,'Subphase_S_components.fig'))
print(4,'-dpng',fullfile(SavePath,'Subphase_S_components.png'))



%% Plot the resulting calibration data z/F and phase angle



figure(5);
subplot(1,2,1)
% plot the amplitude ratio
errorbar(omegaHz,AR_experiment,AR_experiment_std,'-o','Linewidth',2,'MarkerSize',7,'MarkerFaceColor','w'); hold on
    plot(omegaHz,1./abs(Fz_calculated(E,K_compliance)),'--o','Linewidth',2,'MarkerSize',7,'MarkerFaceColor','r'); hold off
set(gca,'XScale','log')
set(gca,'YScale','log')
ylabel('Amplitude Ratio z/I [pixel/Amp]')
xlabel('\omega [Hz]')
legend('measured','fit','Location','best')
set(gca,'FontSize',18)

% plot the phase angles in degrees
angle_calculated = -angle(1./Fz_calculated(E,K_compliance))*180/pi ;
subplot(1,2,2)
errorbar(omegaHz,delta_experiment,delta_experiment_std,'-o','Linewidth',2,'MarkerSize',7,'MarkerFaceColor','w'); hold on
    plot(omegaHz,angle_calculated,'--o','Linewidth',2,'MarkerSize',7,'MarkerFaceColor','r'); hold off
set(gca,'XScale','log')
ylabel('phase shift [deg]')
xlabel('\omega [Hz]')
set(gca,'FontSize',18)

savefig(5,      fullfile(SavePath,'Calibration_AR_delta.fig'))
print(5,'-dpng',fullfile(SavePath,'Calibration_AR_delta.png'))

%% plot the resulting G_s* and eta_s*

Geometry = (Channelradius-Needleradius)/(2*NeedleLength);

G_s_star   = Geometry * Fz_calculated(1,K_compliance);
eta_s_star = Geometry ./(-1i*omegarads) .* (real(Fz_calculated(1,K_compliance))-1i*imag(Fz_calculated(1,K_compliance)));

% plot G*, G' and G''
figure(6)
subplot(2,3,1)
loglog(omegaHz,abs(G_s_star),'--ok','Linewidth',2,'MarkerFaceColor','w')
xlabel('\omega [Hz]')
ylabel('G_{s,SYS}^* [Pa.m]')
set(gca,'FontSize',18)

subplot(2,3,2)
loglog(omegaHz(real(G_s_star)>=0),real(G_s_star(real(G_s_star)>=0)),'--ob','Linewidth',2,'MarkerFaceColor','b'); hold on
loglog(omegaHz(real(G_s_star)<0),-real(G_s_star(real(G_s_star)<0)),'--or','Linewidth',2,'MarkerFaceColor','r'); hold off
legend('positive','negative','Location','best')
xlabel('\omega [Hz]')
ylabel('Real (G_{s,SYS}^*) [Pa.m]')
set(gca,'FontSize',18)

subplot(2,3,3)
loglog(omegaHz,imag(G_s_star),'--ob','Linewidth',2,'MarkerFaceColor','w');
xlabel('\omega [Hz]')
ylabel('Imag (G_{s,SYS}^*) [Pa.m]')
set(gca,'FontSize',18)



% plot eta*, eta' and eta''
subplot(2,3,4)
loglog(omegaHz,abs(eta_s_star),'--sk','Linewidth',2,'MarkerFaceColor','w')
xlabel('\omega [Hz]')
ylabel('\eta_{s,SYS}^* [Pa.m.s]')
set(gca,'FontSize',18)

subplot(2,3,5)
loglog(omegaHz,real(eta_s_star),'--sb','Linewidth',2,'MarkerFaceColor','b');
xlabel('\omega [Hz]')
ylabel('Real (\eta_{s,SYS}^*) [Pa.m.s]')
set(gca,'FontSize',18)

subplot(2,3,6)
loglog(omegaHz(imag(eta_s_star)>=0),imag(eta_s_star(imag(eta_s_star)>=0)),'--sb','Linewidth',2,'MarkerFaceColor','b'); hold on
loglog(omegaHz(imag(eta_s_star)<0),-imag(eta_s_star(imag(eta_s_star)<0)),'--sr','Linewidth',2,'MarkerFaceColor','r'); hold off
legend('positive','negative','Location','best')
xlabel('\omega [Hz]')
ylabel('Imag (\eta_{s,SYS}^*) [Pa.m.s]')
set(gca,'FontSize',18)


savefig(6,      fullfile(SavePath,'Calibration_G_eta_SYS.fig'))
print(6,'-dpng',fullfile(SavePath,'Calibration_G_eta_SYS.png'))


%% Save the data

% Save the calibration parameters
cal.parameters.K_N_m_(1) = K_compliance;
cal.parameters.E_pixelI_div_zF_(1) = E;
cal.parameters.Lambda(1) = lambda;
cal.parameters.Geometry(1) = Geometry;

SIZE = size(cal.data,1)-length(omegaHz);

% save calibration data
cal.data.Omega_Hz_   = [omegaHz; NaN(SIZE,1)];
cal.data.Omega_rads_ = [omegarads; NaN(SIZE,1)];

cal.data.AR_exp_pixel_I_ = [AR_experiment; NaN(SIZE,1)];
cal.data.AR_exp_std_pixel_I_ = [AR_experiment_std; NaN(SIZE,1)];
cal.data.Delta_exp_deg_= [delta_experiment; NaN(SIZE,1)];
cal.data.Delta_exp_std_deg_= [delta_experiment_std; NaN(SIZE,1)];

cal.data.Real_D_sub_Fz_= [real(PARAM.*INTEGRAL); NaN(SIZE,1)];
cal.data.Imag_D_sub_Fz_= [imag(PARAM.*INTEGRAL); NaN(SIZE,1)];
cal.data.K_momega2 = [K_compliance-NeedleMass.*omegarads.^2; NaN(SIZE,1)];

cal.data.ReGstar_Pam_    = [real(G_s_star); NaN(SIZE,1)];
cal.data.ImGstar_Pam_    = [imag(G_s_star); NaN(SIZE,1)];
cal.data.Reetastar_Pams_ = [real(eta_s_star); NaN(SIZE,1)];
cal.data.Imetastar_Pams_ = [imag(eta_s_star); NaN(SIZE,1)];

cal.data.Delta_calc_deg = [angle_calculated; NaN(SIZE,1)];

cal.model = Fz_calculated; 


% Give an output

fprintf('\nThe fitted parameters are:\n E = %10.4e,\n K = %10.4e N/m\n',E,K_compliance);

% TODO
% "nice" plot?
% make option to avoid analytical solution?




end

