function [Data] = ISR_NonLinearCorrection(Data,cal,lim,Sheet)
% Reynaerts analysis to handle the data from the needle rheometer
% The used formalism is Ax = B with x containing the unknown velocities at
% the knods of the discretization scheme


%% Arrange the measured data and parameters for convenience

% Measured Data

frequencyHz = Data.CurrentFrequency_Hz_; % Frequency in rad/s
frequencyrads = frequencyHz*2*pi; % Frequency in rad/s

Objective = Data.Objective_x_;
RelativeMagnification = Objective./cal.parameters.Objective_x_(1);
AR_experimental_pixel_I_ = Data.AmplitudeRatio_pixel_I_./RelativeMagnification; % z/I


% subtract a 20 ms delay
%delay = frequencyrads*0.02;

Delta_experimental_rad_ = Data.MeasuredDelta_deg_*pi/180;%-delay; % deg
%calculate the amplitude ratio of the experiment
Fz_experiment = 1./AR_experimental_pixel_I_.*...
                 (cos(Delta_experimental_rad_) +1i*sin(Delta_experimental_rad_));

% Experimental parameters
Channelradius = (cal.parameters.ChannelWidth_mm_(1)/2 ) *10^(-3); %m
Needleradius = (cal.parameters.NeedleDiameter_mm_(1)/2) *10^(-3); %m
NeedleLength = (cal.parameters.NeedleLength_mm_(1)) *10^(-3); %m
NeedleMass = (cal.parameters.NeedleMass_g_(1)) *10^(-3); % kg
SubphaseDensity = (cal.parameters.SubphaseDensity_g_ml_(1)) * 10^(3); % kg/m^3
SubphaseViscosity = (cal.parameters.SubphaseBulkViscosity_mPas_(1)) *10^(-3); % Pa.s

% calibration parameters
K_compliance = cal.parameters.K_N_m_(1);
E = cal.parameters.E_pixelI_div_zF_(1); % F/z_comp * E = F/z_exp

lambda = cal.parameters.Lambda(1);

% calculate the reynolds number: Re = rho_b * omega * a^2 / eta_b
Re = SubphaseDensity * frequencyrads * Needleradius^2 ./ SubphaseViscosity;

             

% technical parameters for the code

Nexp = length(frequencyHz); % number of experiments
PATH = cal.path;

Nit = 100; % maximum iteration
Nit_min = 50; % minimum iteration

ThM = 64;%39; % mesh size of theta
PM = 64;%39; % mesh size of p = ln(r/a)

channelparameter = log(lambda); % = max p = log (R/a)
dTh = (pi/2) / ThM; % difference in theta which goes from 0 to pi/2: delta Theta
dP = channelparameter / PM; % difference in p which goes from 0 to log(R/r): delta p


% Just define the initial Bo* number as 100.
Bo0 = complex(1-0i)*ones(size(frequencyHz));
Bo(1:Nexp) = struct('Exp',NaN(1,Nit));

% These variables will be used to store the final Bo and status of
% convergence
Bo_end = NaN(size(frequencyHz));
status_converged=zeros(Nexp,1);






%% Begin with the formalism:
        % Step 1: Calculation of the velocity field g
   % Step 2: Calculation of subphase drag on the needle
% Step 3: Comparison of the new amplitude ratio --> change Bo accordingly



% loop over all frequenceis.
for jj = 1:Nexp 
    
% initialise the mesh matrix and the boundary conditions

% A contains all g(theta, p) with 
%       A(1, 1         : ThM+1)   = g(0:pi/2 , 0)
%       A(1, ThM+2     : 2*(ThM+1)) = g(0:pi/2 , 1)
%       A(1, 2*(ThM+2) : 3*(ThM+1)) = g(0:pi/2 , 2)
%       etc.

% Define A0 and b
% A = eye  ((ThM+1)*(PM+1));
b  = zeros((ThM+1)*(PM+1),1);
b(1:ThM+1)=1; % BC at p=0 (eq. 23a)
% create a placeholder for getting the right components to create the
% sparse matrix A
IDX = struct('theta_interface',[],'theta_z',[],'bulk',[]);

for k=((ThM+1)+1):((PM*(ThM+1)))
    
    % This is the number by hich dP has to be multiplied in order to equal the p in the paper
    %p=ceil(k/(PM+1))-1;
    
    % If loop according to the boundary conditions
    if ceil(k/(ThM+1))==floor(k/(ThM+1)) %
        % this case coresponds to the index of A for theta = pi/2 which is
        % the interface plane and corresponds to eq. 24
        
%         A(k,k)         =   2/(dP^2)*(1+2/dTh * Bo0(jj)*exp(-p*dP))  +2/dTh^2 +1i*Re(jj)*exp(2*p*dP);
%         A(k,k+(ThM+1)) = -(1+2/dTh* Bo0(jj)*exp(-p*dP))*(1/dP^2) + Bo0(jj)*exp(-p*dP)/(dP*dTh);
%         A(k,k-(ThM+1)) = -(1+2/dTh* Bo0(jj)*exp(-p*dP))*(1/dP^2) - Bo0(jj)*exp(-p*dP)/(dP*dTh);
%         A(k,k-1)       = -2/dTh^2;
        
        IDX.theta_interface = [IDX.theta_interface ;k];
        
    elseif ceil((k-1)/(ThM+1))==floor((k-1)/(ThM+1))
        % this case corresponds to the index of A for theta = 0 which is
        % the vertical (z axis) and corresponds to eq. 23c
        
        
%         A(k,k)         = (1i*Re(jj)*exp(2*(p)*dP)  +  2/dP^2  +  2/dTh^2);
%         A(k,k+1)       = -2/dTh^2;
%         A(k,k+(ThM+1)) = -1/dP^2;
%         A(k,k-(ThM+1)) = -1/dP^2;
        
        IDX.theta_z = [IDX.theta_z ;k];
        
    else
        % this case corresponds to the bulk equation (eq. 22)
        
%         A(k,k) = (1i*Re(jj)*exp(2*p*dP)   +  2/dP^2   +  2/dTh^2);
%         A(k,k+1) = -1/dTh^2;
%         A(k,k-1) = -1/dTh^2;
%         A(k,k+(ThM+1)) = -1/dP^2;
%         A(k,k-(ThM+1)) = -1/dP^2;
        
        IDX.bulk = [IDX.bulk ;k];
        
    end
end


% use sparse matrices in order to save computation time allow for large mesh sizes.
A_s_0 = sparse(1:(ThM+1)*(PM+1),1:(ThM+1)*(PM+1),1,(ThM+1)*(PM+1),(ThM+1)*(PM+1));

A_s_I1 = sparse(IDX.theta_interface,IDX.theta_interface,          (2/(dP^2)*(1+2/dTh * Bo0(jj)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP))  +2/dTh^2 +1i*Re(jj)*exp(2*(ceil(IDX.theta_interface/(PM+1))-1)*dP)-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_I2 = sparse(IDX.theta_interface,IDX.theta_interface+(ThM+1), -(1+2/dTh* Bo0(jj)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP))*(1/dP^2) + Bo0(jj)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP)/(dP*dTh),     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_I3 = sparse(IDX.theta_interface,IDX.theta_interface-(ThM+1), -(1+2/dTh* Bo0(jj)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP))*(1/dP^2) - Bo0(jj)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP)/(dP*dTh)  ,     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_I4 = sparse(IDX.theta_interface,IDX.theta_interface-1      ,(-2/dTh^2),                             ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));

A_s_Z1 = sparse(IDX.theta_z,IDX.theta_z,(1i*Re(jj)*exp(2*(ceil(IDX.theta_z/(ThM+1))-1)*dP)  +  2/dP^2  +  2/dTh^2-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_Z2 = sparse(IDX.theta_z,IDX.theta_z+1,      (-2/dTh^2),                             ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_Z3 = sparse(IDX.theta_z,IDX.theta_z+(ThM+1),(-1/dP^2),                              ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_Z4 = sparse(IDX.theta_z,IDX.theta_z-(ThM+1),(-1/dP^2),                              ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));

A_s_B1 = sparse(IDX.bulk,IDX.bulk,(1i*Re(jj)*exp(2*(ceil(IDX.bulk/(ThM+1))-1)*dP)   +  2/dP^2   +  2/dTh^2-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_B2 = sparse(IDX.bulk,IDX.bulk+1,(-1/dTh^2),                                     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_B3 = sparse(IDX.bulk,IDX.bulk-1,(-1/dTh^2),                                     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_B4 = sparse(IDX.bulk,IDX.bulk+(ThM+1),(-1/dP^2),                                ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
A_s_B5 = sparse(IDX.bulk,IDX.bulk-(ThM+1),(-1/dP^2),                                ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));

A = A_s_0+...
    A_s_I1+A_s_I2+A_s_I3+A_s_I4+...
    A_s_Z1+A_s_Z2+A_s_Z3+A_s_Z4+...
    A_s_B1+A_s_B2+A_s_B3+A_s_B4+A_s_B5;


% Define constant parameters which are used in the iteration loop
Fz_calculated = complex(NaN(Nit,1)+1i*NaN(Nit,1));
% PARAM & PARAM2 are used for calculating the new amplitude ratio
PARAM  = ( 1i*2*NeedleLength*frequencyrads(jj) * SubphaseViscosity);
PARAM2 = ( K_compliance - NeedleMass*(frequencyrads(jj)^2));  % Compliance + Inertia (with negative sign)

phi = (exp(linspace(0,channelparameter,PM+1))* Needleradius - Needleradius) ./ (Channelradius -Needleradius);

% create the figure which is change lateroin
figure(1)
% evolution of abs(Bo)
subplot(3,2,1) 
pl1 = semilogy(NaN,NaN,'-o','Linewidth',2,'MarkerFaceColor','w','MarkerSize',8);
title(sprintf('Excel Sheet %s Frequency %g of %g',Sheet,jj, Nexp),'Interpreter','none')
xlabel('Iteration')
ylabel('|Bo^*| (-)')
set(gca,'FontSize',16)

% evolution of arg(Bo)
subplot(3,2,3) 
pl3 = plot(NaN,NaN,'-o','Linewidth',2,'MarkerFaceColor','w','MarkerSize',8);
xlabel('Iteration')
ylabel('\theta_s (deg)')
set(gca,'FontSize',16)

% evolution fzrest and conv as a measure of convergence
subplot(3,2,5) 
pl4 = semilogy(NaN,NaN,'-o',NaN,NaN,'-^','MarkerFaceColor','w','Linewidth',2,'MarkerSize',8);
xlabel('Iteration')
ylabel('convergence (-)')
legend('|1-Fz_{calc}/Fz_{exp}|','|1-Bo_n/Bo_{n-1}|')
set(gca,'FontSize',16)

% velocity profile
subplot(3,2,[2 4 6]) 
pl2 = plot(NaN,real(NaN),'-o',...
           NaN,imag(NaN),'-^','MarkerFaceColor','w','Linewidth',2,'MarkerSize',8);
xlim([0 1]);
ylim([-inf,1]);
xlabel('\phi = (r-a) / (R-a)')
ylabel('k(\phi) = g(p,\theta=\pi/2) / g(p=0,\theta=0)')
legend('Real(k(\phi))','Imag(k(\phi))')
set(gca,'FontSize',16)

drawnow
fzrest = NaN(1,Nit);
conv = NaN(1,Nit);
% iterate to find the right Boussinesque number Bo
for it = 1:Nit

    
% update matrix A with new Bo
if it == 1
    % In the first iteration , transfer the matrix A0 to the gpu
    %A=gpuArray(A0); % change variable in order for reducing computation time later on
    Bo(jj).Exp(it)=Bo0(jj);
else
    % update the cells containing Bo in the MAtrix A
    
    A_s_I1 = sparse(IDX.theta_interface,IDX.theta_interface,          (2/(dP^2)*(1+2/dTh * Bo(jj).Exp(it)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP))  +2/dTh^2 +1i*Re(jj)*exp(2*(ceil(IDX.theta_interface/(PM+1))-1)*dP)-1), ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_I2 = sparse(IDX.theta_interface,IDX.theta_interface+(ThM+1), -(1+2/dTh* Bo(jj).Exp(it)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP))*(1/dP^2) + Bo(jj).Exp(it)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP)/(dP*dTh),     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));
    A_s_I3 = sparse(IDX.theta_interface,IDX.theta_interface-(ThM+1), -(1+2/dTh* Bo(jj).Exp(it)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP))*(1/dP^2) - Bo(jj).Exp(it)*exp(-(ceil(IDX.theta_interface/(PM+1))-1)*dP)/(dP*dTh)  ,     ((ThM+1)*(PM+1)),((ThM+1)*(PM+1)));

    
    A = A_s_0+...
    A_s_I1+A_s_I2+A_s_I3+A_s_I4+...
    A_s_Z1+A_s_Z2+A_s_Z3+A_s_Z4+...
    A_s_B1+A_s_B2+A_s_B3+A_s_B4+A_s_B5;
    
%     for idx = 1:length(IDX.theta_interface)
%         k=IDX.theta_interface(idx);
% %         p=ceil(k/(PM+1))-1;
%         
% %         A(k,k)         =   2/(dP^2)*(1+2/dTh * Bo(jj).Exp(it)*exp(-p*dP))  +2/dTh^2 +1i*Re(jj)*exp(2*p*dP);
% %         A(k,k+(ThM+1)) = -(1+2/dTh* Bo(jj).Exp(it)*exp(-p*dP))*(1/dP^2) + Bo0(jj).Exp(it)*exp(-p*dP)/(dP*dTh);
% %         A(k,k-(ThM+1)) = -(1+2/dTh* Bo(jj).Exp(it)*exp(-p*dP))*(1/dP^2) - Bo0(jj).Exp(it)*exp(-p*dP)/(dP*dTh);
% %         A(k,k-1)       = -2/dTh^2;

%     end
end

    

%% STEP 1: Solve the equation for the velocities g with A*g = b
g = full(mldivide(A,sparse(b))); % invert matrix and retreive the result from the gpu
%g = mldivide(A,b);

%% STEP 2: compute Amplitude Ratio of the needle with the computed velocities g


% DERIVATIVE =  -dg/dp at p=0 and theta = pi/2
DERIVATIVE =-( -3*g(ThM+1)+4*g(2*(ThM+1))-g(3*(ThM+1)) )/(2*dP); % g(ThM+1)=1 because of BC

% INTEGRAL = int ( -dg/dp at p=0 over dTheta)
INTEGRAL = -sum( -3*g(ThM+1) + 4*g((ThM+1)+1:2*(ThM+1)) - g(2*(ThM+1)+1:3*(ThM+1)) ) *dTh/(2*dP); %g(1:ThM+1)=1 because of BC


% eq 25 --> calculate AR
Fz_calculated(it) = PARAM*Bo(jj).Exp(it)*DERIVATIVE + ...   % Interfacial contribution
                    PARAM*INTEGRAL + ...                    % Subphase Contribution
                    PARAM2;                                 % Compliance  +  Inertia



%% STEP 3: update the Boussinesq number
% eq.27 --> calculate new Bo

% if it <= Nit_min
%updatemagnitude = abs((Fz_experiment(jj))/(E*Fz_calculated(it)));
%updateangle = angle((Fz_experiment(jj))/(E*Fz_calculated(it)));

%Bo(jj).Exp(it+1)=updatemagnitude*exp(1i*updateangle)*Bo(jj).Exp(it);
    % Stable update criterium
    Bo(jj).Exp(it+1)=(Fz_experiment(jj))/(E*Fz_calculated(it))*Bo(jj).Exp(it);
    % omit negative viscosities
    Bo(jj).Exp(it+1)=abs(real(Bo(jj).Exp(it+1)))+1i*(imag(Bo(jj).Exp(it+1)));
    
    


% plot the evolution of Bo and the velocity profile at delta = pi/2 (on the interface)
% evolution of Bo
set(pl1,'XData',0:it-1);
set(pl1,'YData',abs(Bo(jj).Exp(1:it)));


set(pl3,'XData',0:it-1);
set(pl3,'YData',angle(-imag(Bo(jj).Exp(1:it))+1i*real(Bo(jj).Exp(1:it))).*180/pi);

% velocity profile
K = g((1:PM+1)*(ThM+1))./g(1);

set(pl2(1),'XData',phi);
set(pl2(1),'YData',real(K));
set(pl2(2),'XData',phi);
set(pl2(2),'YData',imag(K));



% calculate the deviation from the calculated to the measured AR
fzrest(it)=abs((E*abs(Fz_calculated(it))-abs(Fz_experiment(jj)))/abs(Fz_experiment(jj)));
conv(it)  =abs(Bo(jj).Exp(it+1)-(Bo(jj).Exp(it)))/abs(Bo(jj).Exp(it));


set(pl4(1),'XData',0:it-1);
set(pl4(1),'YData',fzrest(1:it));
set(pl4(2),'XData',0:it-1);
set(pl4(2),'YData',conv(1:it));
drawnow

% calculate the criterion for convergence
if it>Nit_min % do at minimum 20 calculations
    %conv=abs(Bo(jj).Exp(it+1)-(Bo(jj).Exp(it)))/abs(Bo(jj).Exp(it));
    if conv(it)<lim && fzrest(it) < lim
        % if the mean over the last 10 iterations of Bo is < 10^(-10) and
        % the deviaction between the amplitude ratios is < 10^(-12), then
        % break the iteration
        status_converged(jj) = 1;
        Bo_end(jj)=Bo(jj).Exp(it+1);
        savefig(1,fullfile(PATH,'Analysis',Sheet,sprintf('Sheet_%s_Frequency_%g',Sheet,jj)));
        break;
    elseif it == Nit
        Bo_end(jj)=Bo(jj).Exp(it+1);
        savefig(1,fullfile(PATH,'Analysis',Sheet,sprintf('Sheet_%s_Frequency_%g',Sheet,jj)));
        
    end
elseif it==Nit  
    status_converged(jj) = 0;
    Bo_end(jj)=Bo(jj).Exp(it+1);
    savefig(1,fullfile(PATH,'Analysis',Sheet,sprintf('Sheet_%s_Frequency_%g',Sheet,jj)));
elseif abs(Bo(jj).Exp(it+1))==0
    Bo_end(jj)=Bo(jj).Exp(it+1);
    savefig(1,fullfile(PATH,'Analysis',Sheet,sprintf('Sheet_%s_Frequency_%g',Sheet,jj)));
    break;
end
end
end



%% Store the variables

Data.NonLinBo             =  abs(Bo_end);

Data.NonLinEta_star_Pams_ =  abs(Bo_end) *SubphaseViscosity *Needleradius;
Data.NonLinReEta_Pams_    =  real(Bo_end)*SubphaseViscosity *Needleradius;
Data.NonLinImEta_Pams_    = -imag(Bo_end)*SubphaseViscosity *Needleradius;

Data.NonLinG_star_Pam_    =  abs(Bo_end) *SubphaseViscosity *Needleradius .*frequencyrads;
Data.NonLinReG_Pam_       = -imag(Bo_end)*SubphaseViscosity *Needleradius .*frequencyrads;
Data.NonLinImG_Pam_       =  real(Bo_end)*SubphaseViscosity *Needleradius .*frequencyrads;

Data.NonLinDelta_deg_     =  angle(-imag(Bo_end)+1i*real(Bo_end)) .*180/pi;   %atan(real(Bo_end)./imag(Bo_end)*180/pi);
Data.status_converged     =  status_converged;


